﻿using System;
using System.Threading;

using wclCommon;
using wclSerial;

namespace SerialMonitor
{
    class Program
    {
        private class SerialMonitor
        {
            private ManualResetEvent TerminateEvent;
            private Thread Thread;

            private void Monitor_OnRemoved(Object Sender, wclSerialDevice Device)
            {
                Console.WriteLine("Device REMOVED: ");
                Console.WriteLine("  Friendly name: " + Device.FriendlyName);
                Console.WriteLine("  Is modem: " + Device.IsModem.ToString());
                Console.WriteLine("  Name: " + Device.DeviceName);
            }

            private void Monitor_OnInserted(Object Sender, wclSerialDevice Device)
            {
                Console.WriteLine("Device INSERTED: ");
                Console.WriteLine("  Friendly name: " + Device.FriendlyName);
                Console.WriteLine("  Is modem: " + Device.IsModem.ToString());
                Console.WriteLine("  Name: " + Device.DeviceName);
            }

            private void ThreadProc()
            {
                wclSerialMonitor Monitor = new wclSerialMonitor();
                Monitor.OnInserted += Monitor_OnInserted;
                Monitor.OnRemoved += Monitor_OnRemoved;

                Console.WriteLine("Enumerate installed device.");
                wclSerialDevice[] Devices;
                Int32 Res = Monitor.EnumDevices(out Devices);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Failed to enumerate serial devices: 0x" + Res.ToString("X8"));
                else
                {
                    if (Devices == null || Devices.Length == 0)
                        Console.WriteLine("No serial devices were found");
                    else
                    {
                        int i = 0;
                        foreach (wclSerialDevice Device in Devices)
                        {
                            Console.WriteLine("Device " + i.ToString());
                            Console.WriteLine("  Friendly name: " + Device.FriendlyName);
                            Console.WriteLine("  Is modem: " + Device.IsModem.ToString());
                            Console.WriteLine("  Name: " + Device.DeviceName);
                            i++;
                        }
                    }
                }

                Console.WriteLine("Try to start monitoring");
                Res = Monitor.Start();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Failed to start monitoring serial devices 0x" + Res.ToString("X8"));
                else
                    Console.WriteLine("Monitoring started");

                wclMessageBroadcaster.Wait(TerminateEvent);

                if (Monitor.Monitoring)
                {
                    Console.WriteLine("Try to stop monuioring");
                    Res = Monitor.Stop();
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        Console.WriteLine("Failed to stop monitoring serial devices 0x" + Res.ToString("X8"));
                    else
                        Console.WriteLine("Monitoring stopped");
                }
                Monitor = null;
            }

            public SerialMonitor()
            {
                TerminateEvent = null;
                Thread = null;
            }

            public void Start()
            {
                TerminateEvent = new ManualResetEvent(false);
                Thread = new Thread(ThreadProc);
                Thread.Start();
            }

            public void Stop()
            {
                TerminateEvent.Set();
                Thread.Join();
                Thread = null;

                TerminateEvent.Close();
                TerminateEvent = null;
            }
        };

        static void Main(string[] args)
        {
            wclMessageBroadcaster.SetSyncMethod(wclMessageSynchronizationKind.skApc);

            Console.Clear();
            Console.WriteLine("SERIAL MONITOR DEMO");
            Console.WriteLine("===================");
            Console.WriteLine("The demo shows how to enumerate and monitor serial devices.");

            SerialMonitor Monitor = new SerialMonitor();
            Monitor.Start();
            Console.WriteLine("Press ENTER to stop");
            Console.ReadLine();
            Monitor.Stop();
            Monitor = null;
        }
    }
}