using System;
using System.Text;
using System.Windows.Forms;

using wclSerial;
using wclCommon;

namespace SerialMonitor
{
    public partial class fmMain : Form
    {
        private wclSerialMonitor wclSerialMonitor;
        private wclPowerEventsMonitor FMonitor;

        public fmMain()
        {
            InitializeComponent();
        }

        private void btClear_Click(object sender, EventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void fmMain_Load(object sender, EventArgs e)
        {
            wclSerialMonitor = new wclSerialMonitor();
            wclSerialMonitor.OnInserted += new wclSerialDeviceEvent(wclSerialMonitor_OnInserted);
            wclSerialMonitor.OnRemoved += new wclSerialDeviceEvent(wclSerialMonitor_OnRemoved);

            FMonitor = new wclPowerEventsMonitor();
            FMonitor.OnPowerStateChanged += new wclPowerStateChangedEvent(FMonitor_OnPowerStateChanged);
            FMonitor.Open();
        }

        private Boolean Running = false;

        void FMonitor_OnPowerStateChanged(object Sender, wclPowerState State)
        {
            switch (State)
            {
                case wclPowerState.psResumeAutomatic:
                    if (Running)
                    {
                        Running = false;
                        wclSerialMonitor.Start();
                    }
                    lbEvents.Items.Add("Power changed: psResumeAutomatic");
                    break;

                case wclPowerState.psResume:
                    lbEvents.Items.Add("Power changed: psResume");
                    break;

                case wclPowerState.psSuspend:
                    if (wclSerialMonitor.Monitoring)
                    {
                        Running = true;
                        wclSerialMonitor.Stop();
                    }
                    lbEvents.Items.Add("Power changed: psSuspend");
                    break;

                case wclPowerState.psUnknown:
                    lbEvents.Items.Add("Power changed: psUnknown");
                    break;
            }
        }

        void wclSerialMonitor_OnRemoved(object Sender, wclSerialDevice Device)
        {
            lbEvents.Items.Add("Device removed:");
            lbEvents.Items.Add("Friendly name: " + Device.FriendlyName);
            lbEvents.Items.Add("  Modem: " + Device.IsModem.ToString());
            lbEvents.Items.Add("  DeviceName: " + Device.DeviceName);
        }

        void wclSerialMonitor_OnInserted(object Sender, wclSerialDevice Device)
        {
            lbEvents.Items.Add("Device inserted:");
            lbEvents.Items.Add("Friendly name: " + Device.FriendlyName);
            lbEvents.Items.Add("  Modem: " + Device.IsModem.ToString());
            lbEvents.Items.Add("  DeviceName: " + Device.DeviceName);
        }

        private void fmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            wclSerialMonitor.Stop();
            FMonitor.Close();
        }

        private void btEnum_Click(object sender, EventArgs e)
        {
            lvDevices.Items.Clear();
            
            wclSerialDevice[] Devices;
            Int32 Res = wclSerialMonitor.EnumDevices(out Devices);
            lbEvents.Items.Add("Enumerate serial devices: 0x" + Res.ToString("X8"));

            if (Res == wclErrors.WCL_E_SUCCESS)
            {
                if (Devices != null)
                {
                    for (int i = 0; i < Devices.Length; i++)
                    {
                        ListViewItem Item = lvDevices.Items.Add(Devices[i].FriendlyName);
                        Item.SubItems.Add(Devices[i].IsModem.ToString());
                        Item.SubItems.Add(Devices[i].DeviceName);
                    }
                }
            }
        }

        private void btStart_Click(object sender, EventArgs e)
        {
            Int32 Res = wclSerialMonitor.Start();
            lbEvents.Items.Add("Monitoring started: 0x" + Res.ToString("X8"));
        }

        private void btStop_Click(object sender, EventArgs e)
        {
            Int32 Res = wclSerialMonitor.Stop();
            lbEvents.Items.Add("Monitoring stopped: 0x" + Res.ToString("X8"));
        }
    }
}