using System;
using System.Text;
using System.Windows.Forms;

using wclCommon;
using wclSerial;

namespace UsbMonitor
{
    public partial class fmMain : Form
    {
        private wclUsbMonitor Monitor;

        public fmMain()
        {
            InitializeComponent();
        }

        private void btEnum_Click(object sender, EventArgs e)
        {
            lvDevices.Items.Clear();

            wclUsbDevice[] Devices;
            Int32 Res = Monitor.EnumDevices(out Devices);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Unumeration failed: 0x" + Res.ToString("X8"));
            else
            {
                if (Devices != null)
                {
                    foreach (wclUsbDevice Device in Devices)
                    {
                        ListViewItem Item = lvDevices.Items.Add(Device.FriendlyName);
                        Item.SubItems.Add(Device.HardwareId);
                        Item.SubItems.Add(Device.VendorId.ToString("X4"));
                        Item.SubItems.Add(Device.ProductId.ToString("X4"));
                        Item.SubItems.Add(Device.Revision.ToString("X4"));
                        Item.SubItems.Add(Device.Manufacturer);
                        Item.SubItems.Add(Device.Instance);
                        Item.SubItems.Add(Device.ClassGuid.ToString());
                        Item.SubItems.Add(Device.Enabled.ToString());
                    }
                }
            }
        }

        private void fmMain_Load(object sender, EventArgs e)
        {
            Monitor = new wclUsbMonitor();
            Monitor.OnInserted += new wclUsbDeviceEvent(Monitor_OnInserted);
            Monitor.OnRemoved += new wclUsbDeviceEvent(Monitor_OnRemoved);
        }

        void Monitor_OnRemoved(object Sender, wclUsbDevice Device)
        {
            lbEvents.Items.Add("Device removed: " + Device.Instance);
        }

        void Monitor_OnInserted(object Sender, wclUsbDevice Device)
        {
            lbEvents.Items.Add("Device inserted: " + Device.Instance);
        }

        private void fmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            Monitor.Stop();
        }

        private void btClear_Click(object sender, EventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void btStartMonitoring_Click(object sender, EventArgs e)
        {
            Int32 Res = Monitor.Start();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Start monitoring failed: 0x" + Res.ToString("X8"));
            else
                lbEvents.Items.Add("Monitoring started");
        }

        private void btStopMonitoring_Click(object sender, EventArgs e)
        {
            Int32 Res = Monitor.Stop();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Stop monitoring failed: 0x" + Res.ToString("X8"));
            else
                lbEvents.Items.Add("Monitoring stopped");
        }

        private void btEnable_Click(object sender, EventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                Int32 Res = Monitor.Enable(lvDevices.SelectedItems[0].SubItems[3].Text);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Failed to enable device: 0x" + Res.ToString("X8"));
            }
        }

        private void btDisable_Click(object sender, EventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                Int32 Res = Monitor.Disable(lvDevices.SelectedItems[0].SubItems[3].Text);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Failed to disable device: 0x" + Res.ToString("X8"));
            }
        }
    }
}